﻿(function () {
    "use strict"

    angular.module('starter')
    .factory('cacheLoader', ['$q', 'request', function ($q, request) {
        //{identifierGen:, loadFn:, identifiers: hash of {data:, loaded:, pending: [] } }
        var _cacheMap = {};

        // mainKey: name in string, of the primary key for loader
        // identifierArgs: input for identifierGen
        // loadArgs: input for loadFn
        // refresh: boolean, require new data
        var load = function (options) {
            var mainKey = options.mainKey, identifierArgs = options.identifierArgs, loadArgs = angular.extend(angular.copy(options.loadArgs || {}), { __timestamp: new Date().getTime() }), refresh = options.refresh;
            var loader = _cacheMap[mainKey];
            if (!loader)
                throw new Error('loader not found for mainKey: ' + mainKey);

            var identifier = loader.identifierGen(identifierArgs);
            var cached = loader.cachedItems[identifier];
            if (!cached) {
                cached = { data: null, loaded: false, pending: [] };
                loader.cachedItems[identifier] = cached;
            }

            var defer = new $q.defer();

            if (cached.loaded && !refresh) {
                defer.resolve(cached.data);
            }
            else {
                cached.loaded = false;
                cached.data = null;
                cached.pending.push(defer);
                // if it's the first one, call loadFun
                if (cached.pending.length == 1) {
                    loader.loadFn(loadArgs)
                    .then(function (data) {
                        cached.data = data;
                        cached.loaded = true;
                        while (cached.pending.length > 0) {
                            cached.pending.shift().resolve(data);
                        }
                    })
                    .catch(function (err) {
                        cached.data = null;
                        cached.loaded = false;
                        while (cached.pending.length > 0) {
                            cached.pending.shift().reject(err);
                        }
                    });
                }
            }

            return defer.promise;
        }

        // mainKey: name in string, of the primary key for loader
        // identifierGen: function used to generate secondary key(identifier)
        // loadFn: function used to get data
        var register = function (options) {
            _cacheMap[options.mainKey] = {
                identifierGen: options.identifierGen,
                loadFn: options.loadFn,
                cachedItems: {}
            };
            return ret;
        };

        var dataLoaderFnFactory = function (url) {
            return function (params) {
                var defer = new $q.defer();
                request.get({
                    url: url,
                    params: params,
                    success: function (data) {
                        defer.resolve(data);
                    },
                    error: function (data, status) {
                        console.log(arguments);
                        defer.reject({ data: data, status: status });
                    }
                });
                return defer.promise;
            };
        };

        var ret = {
            load: load,
            register: register,
            //HTTP GET
            dataLoaderFnFactory: dataLoaderFnFactory
        };

        return ret;
    }]);
})();