﻿(function () {
    angular.module('starter')
    .factory('datamodel', ['validate', '$filter', function (validate, $filter) {
        var DataModel = function () {
            this._props = [];
        };
        DataModel.prototype.validate = function (optional) {
            return this._props.every(function (prop) {
                return prop.validate(optional);
            });
        };
        DataModel.prototype.addProp = function (prop) {
            this._props.push(prop);
            return this;
        };
        DataModel.prototype.init = function () {
            this._props.forEach(function (prop) {
                prop.init(); //init prop to 'undefined'/'null'
            });
            return this;
        };

        var ModelProp = function (validator) {
            this.validator = validator;
        };
        ModelProp.prototype.validate = function (optional) {
            return this.validator(this.val, optional);
        };
        ModelProp.prototype.init = function (data) {
            this.val = data;
            return this;
        };


        var pwdValidator = validate.stringLimitGenerator({ max: 16, min: 8 });
        //var bodyValidator = validate.stringLimitGenerator({ min: 1 });
        var UserRegistModel = function () {
            var me = this;
            DataModel.call(me);
            me.mobileProp = new ModelProp(validate.mobile);
            me.verifyCodeProp = new ModelProp(function (input, optional) {
                return optional && validate.nullOrEmpty(input);
            });
            me.passwordProp = new ModelProp(function (input, optional) {
                return Boolean(pwdValidator(input) && (optional && validate.nullOrEmpty(input)));
            });
            me.againProp = new ModelProp(function (input, optional) {
                return Boolean(pwdValidator(input) && (optional && validate.nullOrEmpty(input)));
            });
        };
        UserRegistModel.prototype = Object.create(DataModel.prototype);
        UserRegistModel.prototype.toLO = function () {
            return {
                mobile: this.mobileProp.val,
                verifyCode: this.verifyCodeProp.val,
                password: this.passwordProp.val,
                again: this.againProp.val
            };
        };
        UserRegistModel.prototype.init = function (data) {
            if (!data)
                DataModel.prototype.init.call(this);
            else {
                this.mobileProp.init(data.mobile);
                this.verifyCodeProp.init(data.verifyCode);
                this.passwordProp.init(data.password);
                this.againProp.init(data.again);
            }
            return this;
        };

        return {
            ModelProp: ModelProp,
            DataModel: DataModel,
            UserRegistModel: UserRegistModel
        };
    }]);
})();