﻿(function () {
    "use strict"

    angular.module('starter')
    .factory('main', ['$q', 'url', 'request', 'cacheLoader', 'utility', function ($q, url, request, cacheLoader, utility) {
        var mainFactory = {
            getNews: {
                url: 'http://app.thepaper.cn/clt/jsp/v3/nodeContList.jsp',
                name: 'main.getNews'
            },
            queryPartners: {
                url: url.generate('Admin', 'queryPartners'),
                name: 'main.queryPartners'
            }
        };

        for (var i in mainFactory) {
            var item = mainFactory[i];
            cacheLoader.register({
                mainKey: item.name,
                identifierGen: utility.stringifyParams,
                loadFn: cacheLoader.dataLoaderFnFactory(item.url)
            });
        }

        return {
            //options:
            //refresh
            getNews: function (options) {
                var params = options.data;
                return cacheLoader.load({
                    mainKey: mainFactory.getNews.name,
                    identifierArgs: params,
                    loadArgs: params,
                    refresh: options.refresh
                });
            },

            queryPartners: function (options) {
                var params = { query: options.query };
                return cacheLoader.load({
                    mainKey: mainFactory.queryPartners.name,
                    identifierArgs: params,
                    loadArgs: params,
                    refresh: options.refresh
                });
            },

            //options:
            //refresh
            login: function (options) {
                var defer = new $q.defer();
                request.post({
                    url: 'https://sso.idinfo.cn/v1/tickets',
                    params: {},
                    data: options,
                    success: function (data) {
                        defer.resolve(data);
                    },
                    error: function (data, status) {
                        defer.reject({ data: data, status: status });
                    }
                });
                return defer.promise;
            },
        };
    }]);
})();