﻿(function () {
    "use strict"

    angular.module('starter')
    .factory('utility', ['$q', 'validate', function ($q, validate) {
        var calculateTotalPages = function (pages, pageSize) {
            return Math.ceil(pages / pageSize);
        };

        // sort params and stringify them
        var stringifyParams = function (params) {
            if (!params) {
                return '';
            }

            var ret = [], keys = [];
            for (var key in params) {
                keys.push(key);
                keys.sort();
            }

            for (var i = 0; i < keys.length; ++i) {
                ret.push(keys[i] + '=' + params[keys[i]]);
            }

            return ret.join('&');
        };

        //this is a solution for multi-request, concurrent issue
        var Hub = function () {
            this._cacheMap = {};
        };
        //mainKey: name in string, of the primary key for loader
        //idGen: function used to generate secondary key(idGen)
        //loadFn: function used to get data
        //cache: boolean
        Hub.prototype.register = function (options) {
            this._cacheMap[options.mainKey] = {
                idGen: options.idGen,
                loadFn: options.loadFn,
                items: {},
                cache: options.cache
            };
            return this;
        };
        //mainKey: name in string, of the primary key for loader
        //idArgs: input for idGen
        //loadArgs: input for loadFn
        //refresh: boolean, only applicable when 'cache' is enabled
        Hub.prototype.load = function (options) {
            var defer = new $q.defer();
            var mainKey = options.mainKey, idArgs = options.idArgs, loadArgs = angular.extend(angular.copy(options.loadArgs || {}), { __timestamp: new Date().getTime() }), refresh = options.refresh;
            var loader = this._cacheMap[mainKey];
            if (!loader)
                defer.reject(new Error('loader not found for mainKey: ' + mainKey));
            else {
                var id = loader.idGen ? loader.idGen(idArgs) : '';
                var item = loader.items[id];
                if (!item) {
                    item = { pending: [], loaded: false, cached: null };
                    loader.items[id] = item;
                }
                if (loader.cache && !refresh && item.loaded) {
                    console.log('Hub, data from cache for: ' + mainKey);
                    defer.resolve(item.cached);
                }
                else {
                    item.loaded = false;
                    item.cached = null;
                    item.pending.push(defer);
                    if (item.pending.length == 1) {
                        loader.loadFn(loadArgs)
                        .then(function (data) {
                            if (loader.cache) {
                                item.cached = data;
                                item.loaded = true;
                            }
                            while (item.pending.length > 0)
                                item.pending.shift().resolve(data);
                        })
                        .catch(function (err) {
                            while (item.pending.length > 0)
                                item.pending.shift().reject(err);
                        });
                    }
                }
            }
            return defer.promise;
        };
        //mainKey: name in string, of the primary key for loader
        //idArgs: input for idGen
        //data: data to set
        Hub.prototype.initVal = function (options) {
            var mainKey = options.mainKey, idArgs = options.idArgs;
            var loader = this._cacheMap[mainKey];
            if (!loader)
                throw new Error('loader not found for mainKey: ' + mainKey);
            var id = loader.idGen ? loader.idGen(idArgs) : '';
            if (!loader.items[id])
                loader.items[id] = { pending: [], loaded: true, cached: options.data };
            else
                console.warn('failed to init hub item: already exists, ' + options.mainKey);
        };
        //mainKey: name in string, of the primary key for loader
        Hub.prototype.clear = function (options) {
            var loader = this._cacheMap[options.mainKey];
            if (loader)
                loader.items = {};
        };

        Hub.prototype.clearAll = function () {
            this._cacheMap = {};
        };
        //mainKey: name in string, of the primary key for loader
        //idArgs: input for idGen
        //data: data to set
        Hub.prototype.setVal = function (options) {
            var mainKey = options.mainKey, idArgs = options.idArgs;
            var loader = this._cacheMap[mainKey];
            if (!loader)
                throw new Error('loader not found for mainKey: ' + mainKey);
            var id = loader.idGen ? loader.idGen(idArgs) : '';
            var item = loader.items[id];
            if (item) {
                item.loaded = true;
                item.cached = options.data;
            }
            else
                loader.items[id] = { pending: [], loaded: true, cached: options.data };
        };

        var HTMLElemClearChild = function (elem) {
            while (elem.hasChildNodes()) {
                elem.removeChild(elem.lastChild);
            }
        };

        var HTMLElemAddClass = function (elem, className) {
            var cc = elem.className.split(' ').filter(function (c) { return c != className; });
            cc.push(className);
            elem.className = cc.join(' ');
        };

        var HTMLElemRemoveClass = function (elem, className) {
            elem.className = elem.className.split(' ').filter(function (c) { return c != className; }).join(' ');
        };

        var HTMLElemHasClass = function (elem, classname) {
            var elemClasses = elem.className.split(' ');
            return elemClasses.some(function (c) { return c == classname; });
        };

        var getLongRandomString = function () {
            return new Date().getTime().toString() + Math.ceil(Math.random() * 10000).toString();
        };

        var findFirst = function (arr, fn, def) {
            var l = arr.length;
            for (var i = 0; i < l; ++i) {
                if (fn(arr[i], i))
                    return arr[i];
            };
            return def;
        };

        var generateStringComparer = function (prop, desc) {
            desc = desc ? -1 : 1;
            return function (v1, v2) {
                var prop1 = v1[prop], prop2 = v2[prop];
                if (validate.isString(prop1) && validate.isString(prop2)) {
                    prop1 = prop1.trim().toLowerCase();
                    prop2 = prop2.trim().toLowerCase();
                }
                if (prop1 < prop2)
                    return -1 * desc;
                if (prop1 > prop2)
                    return 1 * desc;
                return 0;
            };
        };

        return {
            calculateTotalPages: calculateTotalPages,
            stringifyParams: stringifyParams,
            Hub: Hub,
            getLongRandomString: getLongRandomString,
            htmlElem: {
                clearChild: HTMLElemClearChild,
                addClass: HTMLElemAddClass,
                removeClass: HTMLElemRemoveClass,
                hasClass: HTMLElemHasClass
            },
            findFirst: findFirst,
            generateStringComparer: generateStringComparer
        };
    }]);
})();