﻿(function () {
    angular.module('starter')
    .factory('validate', [function () {
        var emailRegex = /^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/;
        var mobileRegex = /^1[3|4|5|7|8]\d{9}$/;//手机号码
        var digitsRegex = /^[0-9]{1,15}$/;//数字

        var isString = function (str) {
            return (typeof str) === 'string' || (str instanceof String);
        };
        var isNullOrEmpty = function (input) {
            return input == null || (isString(input) && input.trim() == '');
        };
        return {
            isString: isString,
            nullOrEmpty: isNullOrEmpty,
            email: function (input, optional) {
                return Boolean((isString(input) && emailRegex.test(input.toLowerCase())) || (optional && isNullOrEmpty(input)));
            },
            mobile: function (input, optional) {
                return Boolean((isString(input) && mobileRegex.test(input)) || (optional && isNullOrEmpty(input)));
            },
            password:function (input, optional) {
                var pwdValidator = this.stringLimitGenerator({ max: 16, min: 3 });
                return Boolean(pwdValidator(input) || (optional && isNullOrEmpty(input)));
            },
            valuedString: function (input, optional) {
                return Boolean((isString(input) && input.trim().length > 0) || (optional && isNullOrEmpty(input)));
            },
            number: function (input, optional) {
                if (!input && optional)
                    return true;
                else {
                    var n = Number(input);
                    return !isNaN(n);
                }
            },
            bigInt: function (input, optional) {
                var n = Number(input);
                return !isNaN(n) && Boolean((isString(input) && digitsRegex.test(input.toLowerCase())) || (optional && isNullOrEmpty(input)));
            },
            alwaysTrue: function () { return true; },
            nonEmptyArray: function (input) {
                return Array.isArray(input) && input.length > 0;
            },
            stringLimitGenerator: function (opt) {
                //internal func, no input check
                var max = opt.max, min = opt.min, noTrim = opt.noTrim;
                return function (str) {
                    if (!isString(str))
                        return false;
                    else {
                        var s = noTrim ? str : str.trim();
                        var l = s.length;
                        return (max == null || max >= l) && (min == null || min <= l);
                    }
                };
            }
        };
    }]);
})();